class GraphQLResponse:
    def __init__(self, success: bool, payload) -> None:
        self.success = success
        self.payload = payload

    def __iter__(self):
        yield 'success', self.success
        yield 'payload', self.payload

    def as_dict(self):
        return dict(self)


class GraphQLError:
    def __init__(self, error) -> None:
        self.error = error

    def response(self) -> dict:
        data = {
            'errorMessage': self.get_error_message(),
            'errorType': type(self.error).__name__,
            'errorData': None
        }

        return GraphQLResponse(False, data).as_dict()

    def get_error_message(self):
        return str(self.error)


import base64
import logging
import os
import requests
from jobs_python_lib import constants
from jobs_python_lib.secrets_manager import get_job_service_client_secret
from requests import Timeout

logger = logging.getLogger('jobs_python_lib')
logger.setLevel(logging.INFO)


def validate_authorization(event_context):
    auth_tokens = get_auth_tokens(event_context)
    auth_response = None

    auth_token = auth_tokens[0]

    if auth_token is not None and len(auth_token) > 0:
        auth_response = jwt_authentication(auth_tokens[0])
    elif auth_tokens[1] is not None and len(auth_tokens[1]) > 0:
        auth_response = validate_access_token(auth_tokens[1])

    if auth_response is not None:
        identity_data = get_identity_data(auth_response)  # Not sure if we will even need this for company
        print(identity_data)

    return auth_response


def get_auth_tokens(event_context):
    auth_token = event_context.get('authToken')
    legacy_auth_token = event_context.get('legacyAuthToken')

    return auth_token, legacy_auth_token


def jwt_authentication(jwt):  # pylint: disable=inconsistent-return-statements
    try:
        jwt_url = os.environ[constants.ENV_SECURE_DOMAIN] + constants.JWT_TOKEN_PATH
        data = get_jwt_data(jwt)
        client_id = os.environ[constants.ENV_JOB_SERVICE_CLIENT_ID]
        client_secret = get_job_service_client_secret()
        headers = get_jwt_headers(client_id, client_secret)
        resp = requests.post(jwt_url, data=data, headers=headers)
        if resp.status_code == 200:  # pylint: disable=no-else-return
            return resp.json()
        else:
            raise Exception('Cannot validate jwt access token', resp.json())
    except Timeout:
        logger.error('The request to jwt validation has timed out.')


def get_jwt_data(jwt):
    payload = {
        'grant_type': "federated_login",
        'access_token': jwt
    }
    return payload


def validate_access_token(legacy_access_token):  # pylint: disable=inconsistent-return-statements
    try:
        oauth_url = os.environ[constants.ENV_SECURE_DOMAIN] + constants.VALIDATIONS_PATH + '?' + constants.TOKEN + '=' + legacy_access_token
        resp = requests.get(oauth_url)
        if resp.status_code == 200:
            response_text = resp.json()
            if response_text.get('error') is None:  # pylint: disable=no-else-return
                return response_text
            else:
                raise Exception('Cannot validate legacy access token')
        else:
            raise Exception('Cannot validate legacy access token')
    except Timeout:
        logger.error('The request to oauth token validation has timed out.')


def get_jwt_headers(client_id, client_secret):
    return {
        constants.HEADER_AUTHORIZATION: get_basic_auth_header(client_id, client_secret),
        constants.HEADER_CONTENT_TYPE: constants.CONTENT_TYPE_URLENCODED
    }


def get_basic_auth_header(client_id, client_secret):
    auth_string = client_id + ":" + client_secret
    return "Basic " + str(base64.b64encode(auth_string.encode(constants.CONTENT_ENCODING)), constants.CONTENT_ENCODING)


def get_identity_data(legacy_token_validation_dict):
    company_id = legacy_token_validation_dict.get('company_id')
    user_id = legacy_token_validation_dict.get('user_id')
    user_name = legacy_token_validation_dict.get('username')
    user_type = legacy_token_validation_dict.get('user_type')

    identity_data = {
        'company_id': company_id,
        'user_id': user_id,
        'user_name': user_name,
        'user_type': user_type
    }

    return identity_data



CONTENT_ENCODING = "UTF-8"
CONTENT_TYPE_URLENCODED = "application/x-www-form-urlencoded"
ENV_SECURE_DOMAIN = 'SecureDomain'
ENV_JOB_SERVICE_CLIENT_ID = 'JobsvcClientId'
ENV_JOB_SERVICE_CLIENT_SECRET = 'JobsvcClientSecret'
ENV_FIREHOSE_ACCESS_KEY_ID = 'FirehoseAccessKeyId'
ENV_FIREHOSE_SECRET_KEY_ID = 'FirehoseSecretKeyId'
ENV_IS_KINESIS_PUT = 'IsKinesisPut'
ENV_KINESIS_STREAM_NAME = 'KinesisStreamName'
ENV_COMPLIANCE_EMAIL_ADDRESS = 'ComplianceEmailAddress'
ENV_COMPLIANCE_CC_EMAIL_ADDRESS = 'ComplianceCcEmailAddress'
ENV_COMPLIANCE_EMAIL_FROM_DOMAIN = 'ComplianceEmailFromDomain'
ENV_COMPLIANCE_EMAIL_SUBJECT = 'ComplianceEmailSubject'
ENV_SPARKPOST_SNS_TOPIC_ARN = 'SparkpostTopicArn'
ENV_SPARKPOST_S3_BUCKET_NAME = 'SparkpostS3BucketName'
ENV_IS_CALL_SPARKPOST = 'IsCallSparkpost'
HEADER_AUTHORIZATION = "Authorization"
HEADER_CONTENT_TYPE = "Content-Type"
HEADER_ACCEPT = "Accept"
JWT_TOKEN_PATH = "oauth/token"
TOKEN = "token"
USER_TYPE_CUSTOMER = "customer"
USER_TYPE_TECH_PRO = "techpro"
VALIDATIONS_PATH = "oauth/validations"



from psycopg2.extras import (
    NamedTupleCursor,
)
from psycopg2.pool import SimpleConnectionPool


class PostgresManager:
    """ Postgres connection manager. This class holds the connection pool. Create one per instance"""

    def __init__(self, min_conn: int = 1, max_conn: int = 10, **kwargs):
        self._pool = SimpleConnectionPool(minconn=min_conn, maxconn=max_conn, cursor_factory=NamedTupleCursor, **kwargs)

    def execute(self, *a, **kw):
        """Execute the query and discard the results."""
        with self.get_cursor(*a, **kw):
            pass

    def fetchone(self, *a, **kw):
        """Execute the query and return a single result or None."""
        with self.get_cursor(*a, **kw) as cursor:
            res = cursor.fetchone()
            return res

    def fetchall(self, *a, **kw):
        """Execute the query and yield the results. """
        with self.get_cursor(*a, **kw) as cursor:
            for row in cursor:
                yield row

    def get_cursor(self, *a, **kw):
        """Execute the query and return a context manager wrapping the cursor."""
        return PostgresCursorContextManager(self._pool, *a, **kw)

    def get_transaction(self, *a, **kw):
        """Return a context manager wrapping a transactional cursor."""
        return PostgresTransactionContextManager(self._pool, *a, **kw)

    def get_connection(self):
        """Return a context manager wrapping a connection."""
        return PostgresConnectionContextManager(self._pool)


class PostgresTransactionContextManager:
    """Instantiated once per db.get_transaction call.
    This manager gives you a cursor with autocommit turned off on its
    connection. If the block under management raises then the connection is
    rolled back. Otherwise it's committed. Use this when you want a series of
    statements to be part of one transaction, but you don't need fine-grained
    control over the transaction.
    """

    def __init__(self, pool: SimpleConnectionPool, *a, **kw):  # pylint: disable=unused-argument
        self.pool = pool
        self.conn = None

    def __enter__(self, *a, **kw):
        """Get a connection from the pool.
        """
        self.conn = self.pool.getconn()
        self.conn.autocommit = False
        return self.conn.cursor(*a, **kw)

    def __exit__(self, *exc_info):
        """Put our connection back in the pool.
        """
        if exc_info == (None, None, None):
            self.conn.commit()
        else:
            self.conn.rollback()
        self.conn.autocommit = True
        self.pool.putconn(self.conn)


class PostgresConnectionContextManager:
    """Instantiated once per db.get_connection call.
    This manager turns autocommit off, and back on when you're done with it.
    The connection is rolled back on exit, so be sure to call commit as needed.
    The idea is that you'd use this when you want full fine-grained transaction
    control.
    """

    def __init__(self, pool: SimpleConnectionPool, *a, **kw):  # pylint: disable=unused-argument
        self.pool = pool
        self.conn = None

    def __enter__(self):
        """Get a connection from the pool.
        """
        self.conn = self.pool.getconn()
        self.conn.autocommit = False
        return self.conn

    def __exit__(self, *exc_info):
        """Put our connection back in the pool.
        """
        self.conn.rollback()
        self.conn.autocommit = True
        self.pool.putconn(self.conn)


class PostgresCursorContextManager:
    """Instantiated once per cursor-level db access."""

    def __init__(self, pool: SimpleConnectionPool, *a, **kw):
        self.pool = pool
        self.args = a
        self.kwargs = kw
        self.conn = None

    def __enter__(self):
        """Get a connection from the pool."""
        self.conn = self.pool.getconn()
        cursor = self.conn.cursor()
        try:
            cursor.execute(*self.args, **self.kwargs)
        except Exception:
            self.__exit__()
            raise
        return cursor

    def __exit__(self, *exc_info):
        """Put our connection back in the pool."""
        self.pool.putconn(self.conn)



def find(tree, *keys, include_keys=False, fail_fast=True):
    # pylint: skip-file
    """
    Recursively search for a key in an iterable object

    Args:
        include_keys:
            if false
                return value of keys i.e. ['value1', 'value2', '...']
            if true
                return result as a dict i.e. {'key': 'value'}

        tree: (iterable object):
        keys: (tuple): list of keys
        fail_fast: (bool): Defaults to True
            if True
                A KeyError is raised
            if False
                 returns None

    Returns:
        value: (Any): Key value

    Raises: (KeyError): Key not found
    """

    def tree_traverse(_tree, _key):
        for k, v in _tree.items():
            if k == _key:
                return v
            elif isinstance(v, dict):
                found = tree_traverse(v, _key)
                if found is not None:
                    return found

    results = {} if include_keys else []
    not_found = list()

    for key in keys:
        found_value = tree_traverse(tree, key)

        if found_value is not None or not fail_fast:
            if include_keys:
                results[key] = found_value
            else:
                results.append(found_value)
        else:
            not_found.append(key)

    if fail_fast and not_found:
        raise KeyError('Keys \'{}\' not found'.format(', '.join(not_found)))

    if isinstance(results, list):
        results = tuple(results)

    return results



import json
import os

import boto3

import constants

CONNECTION_POOL = None


def get_job_service_client_secret():
    secrets_client = boto3.client("secretsmanager", region_name='us-east-1')
    secret_name = os.environ[constants.ENV_JOB_SERVICE_CLIENT_SECRET]

    secrets_value_response = secrets_client.get_secret_value(SecretId=secret_name).get('SecretString')
    secret_value = json.loads(secrets_value_response)

    client_secret = secret_value[constants.ENV_JOB_SERVICE_CLIENT_SECRET]

    return client_secret
